@@ -155,6 +155,10 @@ function enigViewSecurityInfo(event, displaySmimeMsg) {
 
         enigHandleUnknownKey();
 
+    } else if (gEnigSecurityInfo.statusFlags & ENIG_MULTIPLE_PARTS) {
+
+      enigDisplayMimeMessage();
+
     } else if ( (gEnigSecurityInfo.statusFlags & nsIEnigmail.NODATA) &&
          (gEnigSecurityInfo.statusFlags &
            (nsIEnigmail.PGP_MIME_SIGNED | nsIEnigmail.PGP_MIME_ENCRYPTED)) ) {
@@ -182,6 +186,58 @@ function enigViewSecurityInfo(event, displaySmimeMsg) {
   }
 }
 
+function enigDisplayMimeMessage() {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigDisplayMimeMessage\n");
+
+  var charset = msgWindow ? msgWindow.mailCharacterSet : "";
+  var contentEncoding = "";
+
+  if ("content-transfer-encoding" in gEnigSavedHeaders) {
+    contentEncoding = gEnigSavedHeaders["content-transfer-encoding"];
+  }
+
+  enigMsgDirect(true, false, contentEncoding, charset, "", 0, "", "", enigDisplayMimeCallback)
+}
+
+function enigDisplayMimeCallback(msgText) {
+  DEBUG_LOG("enigmailMessengerOverlay.js: enigDisplayMimeCallback\n");
+
+  if (!msgText)
+    return;
+
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return;
+
+  var plainText;
+  var exitCode;
+  var statusFlags = 0;
+
+  var errorMsgObj    = new Object();
+  var keyIdObj       = new Object();
+  var exitCodeObj    = new Object();
+  var statusFlagsObj = new Object();
+  var userIdObj      = new Object();
+  var sigDetailsObj  = new Object();
+  var blockSeparationObj = new Object();
+  var signatureObj = new Object();
+  var uiFlags = nsIEnigmail.UI_INTERACTIVE;
+
+  plainText = enigmailSvc.decryptMessage(window, uiFlags, msgText,
+                                signatureObj, exitCodeObj, statusFlagsObj,
+                                keyIdObj, userIdObj, sigDetailsObj, errorMsgObj);
+
+  if (!plainText) {
+     if (interactive && gEnigSecurityInfo && gEnigSecurityInfo.statusInfo)
+       EnigLongAlert(gEnigSecurityInfo.statusInfo);
+     return;
+  }
+
+  // hard-coded to avoid localization for security patch
+  plainText = "Warning: multiple OpenPGP message blocks detected" + "\n\n" + plainText + "\n\n" + "End of first OpenPGP block";
+  EnigLongAlert(EnigGetString("securityInfo")+gEnigSecurityInfo.statusInfo+"\n---------------------------------\n"+plainText);
+}
+
 function enigInitViewHeadersMenu() {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigInitViewHeadersMenu\n");
 
@@ -802,7 +858,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
     if (retry) {
       // Try to verify signature by accessing raw message text directly
       // (avoid recursion by setting retry parameter to false on callback)
-      enigMsgDirect(interactive, importOnly, contentEncoding, charset, newSignature, 0, head, tail);
+      enigMsgDirect(interactive, importOnly, contentEncoding, charset, newSignature, 0, head, tail, enigMessageParseCallback);
       return;
     }
   }
@@ -1309,7 +1365,7 @@ function enigMessageSave() {
   return;
 }
 
-function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signature, bufferSize, head, tail) {
+function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signature, bufferSize, head, tail, callbackFunction) {
   WRITE_LOG("enigmailMessengerOverlay.js: enigMsgDirect: contentEncoding="+contentEncoding+", signature="+signature+"\n");
   var mailNewsUrl = enigGetCurrentMsgUrl();
   if (!mailNewsUrl)
@@ -1333,7 +1389,8 @@ function enigMsgDirect(interactive, importOnly, contentEncoding, charset, signat
                       ipcBuffer:ipcBuffer,
                       expectedBufferSize: bufferSize,
                       head:head,
-                      tail:tail };
+                      tail:tail,
+                      callbackFunction: callbackFunction };
 
   var requestObserver = new EnigRequestObserver(enigMsgDirectCallback,
                                                 callbackArg);
@@ -1401,12 +1458,13 @@ function enigMsgDirectCallback(callbackArg, ctxt) {
                   callbackArg.signature,
                   newBufferSize,
                   callbackArg.head,
-                  callbackArg.tail);
+                  callbackArg.tail,
+                  callbackArg.callbackFunction);
 
   }
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMsgDirectCallback: msgText='"+msgText+"'\n");
 
-  enigMessageParseCallback(msgText, callbackArg.contentEncoding,
+  callbackArg.callbackFunction(msgText, callbackArg.contentEncoding,
                            callbackArg.charset,
                            callbackArg.interactive,
                            callbackArg.importOnly,
